<?php

namespace FiloBlu\Refilo\Plugins\Helper;

use FiloBlu\Refilo\Helper\Data;
use FiloBlu\Refilo\Remote\Entity\UrlRewrite;
use Magento\Framework\App\ResourceConnection;
use Magento\Store\Api\StoreRepositoryInterface;

class Url
{
    /**
     * @var UrlRewrite
     */
    protected $urlRewrite;
    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;
    /**
     * @var Data
     */
    protected $helper;
    /**
     * @var StoreRepositoryInterface
     */
    protected $storeRepository;

    public function __construct(
        Data $helper,
        ResourceConnection $resourceConnection,
        StoreRepositoryInterface $storeRepository
    )
    {
        $this->resourceConnection = $resourceConnection;
        $this->helper = $helper;
        $this->storeRepository = $storeRepository;
    }

    public function beforeGetCanonicalUrl($subject, UrlRewrite $urlRewrite)
    {
        if (!$this->helper->isCanonicalUrlPluginEnabled()) {
            return [$urlRewrite];
        }

        $this->urlRewrite = $urlRewrite;

        return [$urlRewrite];
    }
    public function beforeGetStoreUrl($subject, $storeId, string $path = null)
    {
        if (!$this->helper->isCanonicalUrlPluginEnabled()) {
            return [$storeId, $path];
        }

        if (empty($this->urlRewrite)) {
            return [$storeId, $path];
        }

        if (!in_array($this->urlRewrite->getType(), ['cms-page', 'category', 'product'])) {
            return [$storeId, $path];
        }

        $store = $this->storeRepository->getById($storeId);;

        $connection = $this->resourceConnection->getConnection();
        $storeCode = $store->getCode(); // $this->urlRewrite->getStore();
        $identifier = $this->urlRewrite->getPageIdentifier();
        $targetUrl = $this->urlRewrite->getTargetUrl();


        //This query will returns request_path from filoblu_refilo_redirect if exists
        //otherwise it will return original request_path
        $select = $connection->select()->from(['u' => 'url_rewrite'], ['request_path' => "IF (frr.target_url = '' OR frr.target_url IS NULL, `u`.`request_path`, frr.target_url)"]);
        $select->joinLeft(
            ['frr' => 'filoblu_refilo_redirect'],
            'frr.request_url like CONCAT("%'. $storeCode .'/", u.request_path, "%")',
            []
        );
        $select->order('frr.id DESC');
        $select->limit(1);


        $trail = '';
        switch ($this->urlRewrite->getType()) {
            case 'cms-page':
                $trail = '/';
                $select->joinRight(
                    ['cp' => 'cms_page'],
                    'cp.page_id = u.entity_id AND entity_type = \'cms-page\'',
                    ['refilo_page_identifier' => 'cp.refilo_page_identifier']
                )->where(
                    "COALESCE (
                    IF(cp.refilo_page_identifier = '' OR cp.refilo_page_identifier IS NULL, NULL, cp.refilo_page_identifier),
                    IF(cp.identifier = '' OR cp.identifier IS NULL, NULL, cp.identifier)
                    ) = '{$identifier}'"
                )->where(
                    "u.target_path = '$targetUrl'",
                );
                break;
            case 'category':
                $select->where("u.entity_type = 'category' AND target_path = '$targetUrl'");
                break;
            case 'product':
                $select->where("u.entity_type = 'product' AND target_path = '$targetUrl'");
                break;
            case 'master_data':
                $select->where("u.entity_type = 'contenttype_content' AND target_path = '$targetUrl'");
                break;
            default:
                return null;
        }

        $select->where('u.store_id = ? AND u.is_autogenerated = 1', $storeId);
        $path = $connection->fetchOne($select);

        $path = $this->clearPath($path, $storeCode);

        return [$storeId, $path];
    }

    /**
     * Cleaning path from domain/storecode/ string since it will be added
     * later in getStoreUrl method in order to get a clean canonical url
     *
     * @param $path
     * @param $storeCode
     * @return mixed|string
     */
    private function clearPath($path, $storeCode)
    {
        if (preg_match("/^\/{$storeCode}\/(.*?)\/?$/im", $path, $matches)) {
            return $matches[1];
        }

        if (preg_match("/\.*\/{$storeCode}\/(.*?)\/?$/im", $path, $matches)) {
            return $matches[1];
        }

        return $path;
    }
}
